package models

import (
	"eta/eta_task/global"
	"eta/eta_task/utils"
	"gorm.io/gorm"
	"time"
)

type Report struct {
	Id                 int       `gorm:"column:id;primaryKey;autoIncrement" description:"报告Id"`
	AddType            int       `description:"新增方式：1:新增报告，2:继承报告"`
	ClassifyIdFirst    int       `description:"一级分类id"`
	ClassifyNameFirst  string    `description:"一级分类名称"`
	ClassifyIdSecond   int       `description:"二级分类id"`
	ClassifyNameSecond string    `description:"二级分类名称"`
	Title              string    `description:"标题"`
	Abstract           string    `description:"摘要"`
	Author             string    `description:"作者"`
	Frequency          string    `description:"频度"`
	CreateTime         string    `description:"创建时间"`
	ModifyTime         time.Time `description:"修改时间"`
	State              int       `description:"1:未发布，2:已发布"`
	PublishTime        time.Time `description:"发布时间"`
	Stage              int       `description:"期数"`
	MsgIsSend          int       `description:"消息是否已发送，0:否，1:是"`
	Content            string    `description:"内容"`
	VideoUrl           string    `description:"音频文件URL"`
	VideoName          string    `description:"音频文件名称"`
	VideoPlaySeconds   string    `description:"音频播放时长"`
	VideoSize          string    `description:"音频文件大小，单位M"`
	ContentSub         string    `description:"内容前两个章节"`
	HasChapter         int       `description:"是否有章节 0-否 1-是"`
	ChapterType        string    `description:"章节类型 day-晨报 week-周报"`
	OldReportId        int       `description:"research_report表ID(后续一两个版本过渡需要,之后可移除)"`
	PreMsgSend         int       `description:"定时发布成功后是否立即推送模版消息：0否，1是"`
	AdminId            int
	AdminName          string `description:"系统用户名称"`
	ClassifyIdThird    int    `description:"三级分类id"`
	ClassifyNameThird  string `description:"三级分类名称"`
}

func (m *Report) AfterFind(db *gorm.DB) (err error) {
	m.CreateTime = utils.GormDateStrToDateTimeStr(m.CreateTime)
	return
}

// GetPrePublishedReports 获取定时发布时间为当前时间的未发布的报告列表
func GetPrePublishedReports(startTime, endTime, afterDate string) (list []*Report, err error) {
	o := global.DbMap[utils.DbNameReport]
	sql := `SELECT * FROM report WHERE state = 1 and pre_publish_time >= ? and pre_publish_time <=? and modify_time >= ?`
	err = o.Raw(sql, startTime, endTime, afterDate).Find(&list).Error
	return
}

// PublishReportById 发布报告
func PublishReportById(reportId int, publishTime time.Time) (err error) {
	o := global.DbMap[utils.DbNameReport]
	sql := `UPDATE report SET state = 2, publish_time = ?, modify_time = NOW() WHERE id = ? `
	err = o.Exec(sql, publishTime, reportId).Error
	return
}

func ModifyReportVideo(reportId int, videoUrl, videoName, videoSize string, playSeconds float64) (err error) {
	o := global.DbMap[utils.DbNameReport]
	sql := `UPDATE report SET video_url=?,video_name=?,video_play_seconds=?,video_size=? WHERE id=? `
	err = o.Exec(sql, videoUrl, videoName, playSeconds, videoSize, reportId).Error
	return
}

// ModifyReportVideoByNoVideo
// @Description: 修改无音频的报告音频信息
// @author: Roc
// @datetime 2024-07-25 18:03:05
// @param reportId int
// @param videoUrl string
// @param videoName string
// @param videoSize string
// @param playSeconds float64
// @return err error
func ModifyReportVideoByNoVideo(reportId int, videoUrl, videoName, videoSize string, playSeconds float64) (err error) {
	o := global.DbMap[utils.DbNameReport]
	sql := `UPDATE report SET video_url=?,video_name=?,video_play_seconds=?,video_size=? WHERE id=? AND video_url=""`
	err = o.Exec(sql, videoUrl, videoName, playSeconds, videoSize, reportId).Error
	return
}

type ElasticReportDetail struct {
	ReportId           int    `description:"报告ID"`
	ReportChapterId    int    `description:"报告章节ID"`
	Title              string `description:"标题"`
	Abstract           string `description:"摘要"`
	BodyContent        string `description:"内容"`
	PublishTime        string `description:"发布时间"`
	PublishState       int    `description:"发布状态 1-未发布 2-已发布"`
	Author             string `description:"作者"`
	ClassifyIdFirst    int    `description:"一级分类ID"`
	ClassifyNameFirst  string `description:"一级分类名称"`
	ClassifyIdSecond   int    `description:"二级分类ID"`
	ClassifyNameSecond string `description:"二级分类名称"`
	ClassifyId         int    `description:"最小单元的分类ID"`
	ClassifyName       string `description:"最小单元的分类名称"`
	Categories         string `description:"关联的品种名称(包括品种别名)"`
	StageStr           string `description:"报告期数"`
}

type ChartPermissionMappingIdName struct {
	PermissionId   int
	PermissionName string
}

func GetChartPermissionNameFromMappingByKeyword(source string, classifyId int) (list []*ChartPermissionMappingIdName, err error) {
	o := global.DbMap[utils.DbNameReport]
	sql := " SELECT b.chart_permission_id AS permission_id,b.permission_name FROM chart_permission_search_key_word_mapping AS a INNER JOIN chart_permission AS b ON a.chart_permission_id = b.chart_permission_id WHERE a.from = ? AND a.classify_id = ? "
	sql = utils.ReplaceDriverKeywords("", sql)
	err = o.Raw(sql, source, classifyId).Find(&list).Error
	return
}

func UpdateReportPublishTime(reportId int, videoNameDate string) (err error) {
	o := global.DbMap[utils.DbNameReport]
	sql1 := ` UPDATE report SET publish_time = NOW() WHERE id = ?  `
	err = o.Exec(sql1, reportId).Error
	if err != nil {
		return
	}
	//修改音频标题
	sql2 := ` UPDATE report SET video_name=CONCAT(SUBSTRING_INDEX(video_name,"(",1),"` + videoNameDate + `") WHERE id = ? and (video_name !="" and video_name is not null)`
	err = o.Exec(sql2, reportId).Error
	return
}

func UpdateReportChapterPublishTime(reportId int, videoNameDate string) (err error) {
	o := global.DbMap[utils.DbNameReport]
	sql1 := ` UPDATE report_chapter SET publish_time = NOW() WHERE report_id = ? `
	err = o.Exec(sql1, reportId).Error
	if err != nil {
		return
	}
	//修改音频标题
	sql2 := ` UPDATE report_chapter SET video_name=CONCAT(SUBSTRING_INDEX(video_name,"(",1),"` + videoNameDate + `") WHERE report_id = ? and (video_name !="" and video_name is not null)`
	err = o.Exec(sql2, reportId).Error
	return
}

func ModifyReportMsgIsSend(reportId int) (err error) {
	o := global.DbMap[utils.DbNameReport]
	sql := `UPDATE report SET msg_is_send = 1, msg_send_time=NOW()  WHERE id = ? and msg_is_send=0`
	err = o.Exec(sql, reportId).Error
	return
}

// ClearReportSaveLog 清理报告保存日志
func ClearReportSaveLog(date string) (err error) {
	o := global.DbMap[utils.DbNameReport]
	sql := `DELETE FROM report_save_log WHERE create_time <= ?`
	err = o.Exec(sql, date).Error
	return
}

// PublishReportAndChapter 发布报告及章节
func PublishReportAndChapter(reportInfo *Report, isPublishReport bool, cols []string) (err error) {
	o := global.DbMap[utils.DbNameReport]
	to := o.Begin()
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()
	// 更新报告
	if isPublishReport {
		err = to.Select(cols).Updates(reportInfo).Error
		if err != nil {
			return
		}
	}

	// 发布该报告的所有章节
	sql := ` UPDATE report_chapter SET publish_state = 2, publish_time = ? WHERE report_id = ?  `
	err = to.Exec(sql, reportInfo.PublishTime, reportInfo.Id).Error

	// 发布章节
	//if len(publishIds) > 0 {
	//	sql := ` UPDATE report_chapter SET publish_state = 2, publish_time = ? WHERE report_id = ? AND report_chapter_id IN (` + utils.GetOrmInReplace(len(publishIds)) + `) `
	//	err = to.Raw(sql, reportInfo.PublishTime, reportInfo.Id, publishIds).Error
	//}

	//if len(unPublishIds) > 0 {
	//	sql := ` UPDATE report_chapter SET publish_state = 1, publish_time = NULL, is_edit = 0 WHERE report_id = ? AND report_chapter_id IN (` + utils.GetOrmInReplace(len(unPublishIds)) + `) `
	//	err = to.Raw(sql, reportInfo.Id, unPublishIds).Error
	//}

	return
}
